@@ -5,7 +5,7 @@
   #x <- installed.packages()
   #cat(pkgname, x[x[,1]==pkgname,3], "(2006)\n")
   #rm(x)
-  cat("ks 1.4.10 (2007)\n")
+  #cat("ks 1.4.10 (2007)\n")
 }
 
 
